"use strict";
const baqend = require('baqend');
const args = require('./args');

let globalDb, emf;

const isInitialized = () => {
  return !!emf;
};

const initialize = (req) => {
  emf = new baqend.EntityManagerFactory({
    host: args.host,
    schema: req.body,
    tokenStorage: new baqend.util.TokenStorage()
  });

  globalDb = emf.createEntityManager();
  setUserData(globalDb, req);
};

const getGlobalDB = () => {
  return globalDb;
};

/**
 * Create a anonymous db that have node permission
 * @return {*}
 */
const createNodeDBForJob = () => {
  const db = emf.createEntityManager();
  db.token = globalDb.token;
  return db;
};

/**
 * Create a db for the logged in user with additional node permission
 * @return {EntityManager}
 */
const createUserDBForRequest = (req) => {
  const db = emf.createEntityManager();
  setUserData(db, req);
  return db;
};

const logError = (e) => {
  if (globalDb) {
    return globalDb.log.error(e && e.stack ? e.stack : e).catch(function(err) {
      console.error(err);
      throw err;
    });
  } else {
    console.error(e);
    return Promise.reject(e);
  }
};

const setUserData = (db, req) => {
  db.token = req.token;
  if (req.userId) {
    db.me = db.getReference('User', req.userId);
    db.me.roles = !req.roleIds ? [] : req.roleIds.map(function(id) {
      return db.getReference('Role', id);
    });
  }
};

exports.initialize = initialize;
exports.isInitialized = isInitialized;
exports.getGlobalDB = getGlobalDB;
exports.createUserDBForRequest = createUserDBForRequest;
exports.createNodeDBForJob = createNodeDBForJob;
exports.logError = logError;