"use strict";

const path = require('path');

const exclude = ['status', 'message', 'stack', 'cause'];

function createError(e, debug) {
  const json = serializeError(e, debug);
  json.status = e.status || 500;
  return json;
}

function serializeError(e, debug) {
  const json = {
    message: e.message
  };

  if (e.reason)
    json.reason = e.reason;

  if (debug) {
    json.className = e.name;
    json.stackTrace = serializeStackTrace(e.stack);

    if (e.cause) {
      json.cause = serializeError(e.cause);
    }
  }

  for (let name in e) {
    if (e.hasOwnProperty(name) && exclude.indexOf(name) == -1) {
      json[name] = e[name];
    }
  }

  return json;
}

function serializeStackTrace(stack) {
  const trace = [];
  const lines = stack ? stack.split('\n') : [];
  const linePattern = /at ([^\(]*) \((.*, )?(.+):(\d+):\d+\)/;

  //skip first line
  let i = 1, line;
  for (; line = lines[i]; ++i) {
    let parts = linePattern.exec(line);
    if (!parts)
      continue;

    let className, fileName;
    const methodName = parts[1], lineNumber = Number(parts[4]);
    fileName = path.basename(parts[3]);
    className = path.basename(fileName, path.extname(fileName));

    if (parts[3].indexOf('/code/') == 0) {
      fileName = parts[3];
    }

    trace.push({
      fileName: fileName,
      className: className,
      methodName: methodName,
      lineNumber: lineNumber
    });
  }

  return trace;
}

module.exports = createError;