#!/usr/bin/env node
"use strict";

const express = require('express');
const bodyParser = require('body-parser');
const compression = require('compression');
const errorHandler = require('./helper/error');
const net = require('./helper/net');
const Db = require('./helper/db');
const app = express();
const args = require('./helper/args');

app.use(compression());
app.use(bodyParser.text({limit: '16mb', type: 'application/javascript'}));
app.use(bodyParser.text({limit: '16mb', type: 'text/html'}));
app.use(bodyParser.json({limit: '16mb'}));
app.use(bodyParser.text({limit: '16mb'}));
app.use(bodyParser.urlencoded({limit: '16mb', extended: false}));

const code = require('./routes/code');
app.use('', code);
app.use('', require('./routes/proof'));

app.set('trust proxy', true);
app.disable('x-powered-by');

app.use(function(err, req, res, next) {
  if (!res.headersSent) {
    const jsonError = errorHandler(err, args.debug);
    if (jsonError.status >= 500) {
      Db.logError(err);
    }

    res.set('Content-Type', 'application/json;charset=utf-8');
    res.status(jsonError.status);
    res.send(jsonError);
  } else {
    Db.logError(err);
  }
});

process.on('uncaughtException', function(e) {
  console.error(e);
  Db.logError(e).then(function() {
    process.exit(1);
  }, function() {
    process.exit(1);
  });
});

process.on('unhandledRejection', function(reason, p) {
  console.error(reason);
  Db.logError(reason);
});

express.request.__proto__ = net.HttpRequest.prototype;
express.response.__proto__ = net.HttpResponse.prototype;

const server = new net.Server(app);

server.listen(args.baqendUrl, () => {
  console.log("node.js worker connected to: " + args.baqendUrl);
});



